%%%% This code computes the ratio of the cumulative IRF of a Calvo+ model
%%%% to its approximation using the first eigenfunction-eigenvalue with
%%%% non-zero projection. 

%%%% It computes this ratio as a function of one parameter, the ratio of
%%%% price adjustment a la Calvo to the total number of price adjustments.

clear all;
close all;

phi_vec = [0.001:0.001:10];
phi_vec = phi_vec.^2;

% ell_vec = vector of values for ratio zeta/N_a.
ell_vec  = phi_vec*0 ;       % ratio of Calvo adj relative to total adjustment
M_cal_Kurt_vec = phi_vec*0 ; % cumulatve IRF, formula in AER
M_cal_2_vec = phi_vec*0 ;    % cumulative IRF based on second eigenvalue
b2_vec = phi_vec*0 ;         % coefficient of second eigenvalue
lambda2_vec =  phi_vec*0 ;   % second eigenvalue
ratio_vec = phi_vec*0;
slope_IRF_plan_vec = phi_vec*0 ;
M_b0_vec = phi_vec*0 ;

M=10000;
j_vec = [2:4:M]; 

for i=1:length(phi_vec)
    
    phi= phi_vec(i);
    s = sqrt(2*phi) ;
     
    % ell = z/N_a
    ell_vec(i) = (cosh(s)-1)/cosh(s) ;
    
    % cumulative IRF, for N_a=1 based on Kurtosis formula from AER
    M_cal_Kur_vec(i) = ( exp(s) + exp(-s) )/(( exp(s) + exp(-s) -2 )^2) * ( exp(s)+exp(-s)-2-2*phi) ;
    
    % first non-zero coefficient
    b2_vec(i) = (1+cosh(s))/(cosh(s)-1) * (8*s^2)/(4*s^2+(2*pi)^2) ; 
    
    % second eigenvalue
    % lambda2 = zeta (1 + (2 pi)^2/8 /phi) and N = zeta  / ell => 
    % lambda2 = N ell (1 + (2 pi)^2/8 /phi)
    % so for N = 1:
    lambda2_vec(i) = - ell_vec(i) *   (1 + (2 * pi)^2/8 /phi ) ;
    
    %%%% Cumulative IRF using second eigenvalue
    M_cal_2_vec(i) = b2_vec(i)/(- lambda2_vec(i)) ;
    
     
    ratio_vec(i)  = 2 * (1+ cosh(s)) / ( (cosh(s)-1-phi) * ( 1 + pi^2/(2*phi))^2 );  
    
    rho = (sinh(s)-s)/(s*(cosh(s)-1));
    slope_IRF_plan_vec(i) = -2 * (( 1 + cosh(s) )/( cosh(s)-1 ) * ( ( s*cosh(s) - 2*sinh(s) + s) /(s*(cosh(s)-1)) ) - 1) ;
    % - 2 * ((1+cosh(s))/(cosh(s)-1)*(1 - 2*rho) -1) ;
    
    % M_0 = sum b^0_j lambda_j exp(lambda_j t)  
    lambda_vec = ell_vec(i) * (1+ j_vec.^2*pi^2/(8*phi)); 
    b_0_vec = - 4*rho*(1+cosh(s))/(cosh(s)-1)  * 1./(1+ j_vec.^2*pi^2/(8*phi));
    M_b0_vec(i) = sum( b_0_vec ./ lambda_vec ) ;
     
    
end


%%%%%%% FIGURES %%%%%%

l1l2 = (8*phi_vec+ pi^2)./ (8*phi_vec+ 4* pi^2) ;

figure(1);
ph=plot(ell_vec,1./l1l2 );set(ph, 'linewidth',2)
th=title('Dominant ($\lambda_1$) vs Leading ($\lambda_2$) eigenvalue in Calvo$^+$ model');
set(th,'Interpreter','Latex','Fontsize',22)
xh=xlabel('Calvoness: $\ell(\phi)$');
set(xh,'Interpreter','Latex','Fontsize',18)
yh=ylabel('Selection: $\lambda_2(\phi)/\lambda_1(\phi)$');
set(yh,'Interpreter','Latex','Fontsize',22)
lh=legend('$\frac{\lambda_2(\phi)}{\lambda_1(\phi)}= \frac{  Asympt-half-life-price-changes}{ Asympt-half-life-output  }$ ');
set(lh,'Interpreter','Latex','Fontsize',22)

% % print -depsc Fig_selection_ell
% % print -dpdf Fig_selection_ell


cosh_vec=cosh(sqrt(2*phi_vec));
cum_output_N1 = cosh_vec.*(cosh_vec-1-phi_vec)./(cosh_vec-1).^2;

figure(2);
ph=plot(ell_vec, cum_output_N1 );set(ph, 'linewidth',2)
th=title('Cumulative  output after small shock in Calvo$^+$ model');
set(th,'Interpreter','Latex','Fontsize',22)
xh=xlabel('Calvoness: $\ell(\phi)$');
set(xh,'Interpreter','Latex','Fontsize',18)
yh=ylabel('Cumulative output');
set(yh,'Interpreter','Latex','Fontsize',22)

% print -depsc Fig_cum_output_ell
% print -dpdf Fig_cum_output_ell
