%%%% Script to produce graph of IRF of Price pLans

% This file computes the impulse response of a monetary shock using
% eigenfunctions-eigenvalues in the Price Plan - Calvo plus model

clear all ;
close all ;


M = 10000 ; % number of eigenfunctions-eigenvalues to use

%%% Parameters for Model:

ell_target = [0.70 0.95] ;
sigma = 0.1 ;

figure(1);

for i=1:length(ell_target)
    
    ell = ell_target(i);
    
    s = fzero(@(s) fct_zero_ell(s, ell), ell^2/2) ;
    
    s_vec(i)        = s ;
    phi_vec(i)      = s^2/2 ;
    ell_vec(i)      = (cosh(s)-1)/cosh(s) ; % check ell = z/N
    rho_vec(i)      = (exp(s) - exp(-s) - 2*s) /(s * ( exp(s) + exp(-s) - 2 )) ;
    zeta_vec(i)     = ell_vec(i) ; % inforce N=1;
    theta_vec(i)    = sqrt(2*zeta_vec(i)/sigma^2);
    xbar_vec(i)     = s_vec(i)/theta_vec(i);
    xtilde_vec(i)   = rho_vec(i) * xbar_vec(i) ; %% threshold for plans
    
end


for r=1:length(ell_target)
    
    phi     = phi_vec(r);
    s       = s_vec(r);
    ell     = ell_vec(r);
    rho     = rho_vec(r);
    xtilde  = xtilde_vec(r) ;
    theta   = theta_vec(r) ;
    zeta    = zeta_vec(r);
    xbar    = xbar_vec(r);
      
    %%%%% Coefficients for IRF: product of the coefficients %%%%%%%
    
    b_vec = zeros(1,M) ;
    i=0;
    j=2+4*i;
    while j<= M
        b_vec(j) = (1+cosh(sqrt(2*phi)))/(cosh(sqrt(2*phi))-1) * 2 /(1 + j^2 * pi^2/(8*phi));
        %b_vec(j) = (1+cosh(theta*xbar))/(cosh(xbar*theta)-1)* (8*theta^2*xbar^2)/(4*theta^2*xbar^2+ j^2*pi^2);
        i=i+1;
        j=2+4*i;
    end
    i=0;
    j=4+4*i;
    while j<= M
        b_vec(j) = - 2/(1 + j^2*pi^2/(8*phi));
        %b_vec(j) = - (8*theta^2*xbar^2)/(4*theta^2*xbar^2 + j^2*pi^2);
        i=i+1;
        j=4+4*i;
    end
    
    b_0_vec = zeros(1,M);
    i=0;
    j=2+4*i;
    while j<= M
        b_0_vec(j) = -(4*rho)*(1+cosh(sqrt(2*phi)))/(cosh(sqrt(2*phi))-1)* 1 /(1 + j^2*pi^2/(8*phi));
        %b_0_vec(j) = -(8*rho)*(1+cosh(theta*xbar))/(cosh(xbar*theta)-1)* (2*theta^2*xbar^2)/(4*theta^2*xbar^2+ j^2*pi^2);
        i=i+1;
        j=2+4*i;
    end
    
    
    %%%%% lambda_vec: is the vector of eigenvalues
    
    lambda_vec = zeros(1,M);
    for j=1:M
        lambda_vec(j) = - zeta - sigma^2/xbar^2/8*(pi*j)^2 ;
    end
    
    %%% Number of adjustment per unit of time:
    N_a(r) = zeta * cosh(theta*xbar)/(cosh(theta*xbar)-1);
    %
    
    %%%% Computation of IRF using Fourier series:
    dt=1/(24*360);
    t_vec = [0:dt:10/N_a(r)] ;
    L = length(ell_target);
  %  F_vec = zeros(L,length(t_vec));
  %  F_0_vec = zeros(L,length(t_vec));
    
    for i=1:length(t_vec)
        t = t_vec(i) ;
        
        F_vec(r,i)   = sum(b_vec .* exp(t*lambda_vec)) ;
        F_0_vec(r,i) = sum(b_0_vec .* exp(t*lambda_vec)) ;
        
        %F_2_vec(r,i) = (b_vec(2)+b_0_vec(2)) * exp(t*lambda_vec(2)) ;
        %F_4_vec(r,i) = (b_vec(4)+b_0_vec(4)) * exp(t*lambda_vec(4)) ;
    end
    
    %%% F_vec: impulse response based on M terms in Fourier series
    %%% F_2_vec: impulse response based on first term with non-zero projection
    %%% on eigen-function.
    
    hold on
    ph=plot(t_vec, F_vec(r,:)+F_0_vec(r,:), 'k');
    set(ph,'linewidth',2)
    
    [value, t_i] = max(F_vec(r,:)+F_0_vec(r,:));
    
    tx = text(t_vec(t_i), F_vec(r,t_i)+F_0_vec(r,t_i), ['$\ell =  $ ', num2str(ell_vec(r),3)] );
    set(tx,'Interpreter','Latex','Fontsize',16);

    
end

ti=title('Output IRF to monetary shock, Price Plan Model') ;
xl = xlabel('time');
yl = ylabel('$Y(t)$');
set(yl,'Interpreter','Latex','Fontsize',20);
set(ti,'Interpreter','Latex','Fontsize',20);
set(xl,'Interpreter','Latex','Fontsize',20);
vline(1/N_a(r)) ;
axis([-0.005/N_a(r) 3/N_a(r) 0 max(max(F_vec+F_0_vec))*1.1]) ;
hold off ;

% print -depsc Fig_IRF_Plans_hump
% print -dpdf Fig_IRF_Plans_hump


