%%%%% Computes analytic IRF  using Discrete-time discrete-state  method 
% assumes symmetric problem

clear all;
close all;

% Generalized  Hazard function: specify the  desired form of Lambda(x) 

% quadratic: Lambda(x) = Lambda_0 + Lambda_2 *x^nu;
Lambda_0 =  0.0  ;
nu       =  2;
Lambda_2  = 90;

% BM innovations
sigma    = 0.20;
X        = 2   ; %%% barrier

% number of states in positive values
m = 1000; 
K = m*2+1;
ms = m+1; % mstar reinjection location

dx = X/m;
Delta = dx^2/sigma^2;
x_vec=[-dx*m:dx:dx*m]';

% Compute  Lambda(x)
Lambda_vec = Lambda_0 + Lambda_2 * abs(x_vec).^nu ;

%%% Transition matrix  Mbar

Mbar  = zeros(K,K);

%%% p_{t+Delta}(k) = 1/2 p(k-1)(1-Lambda(k+1)) + 1/2 p(k-1)(1-Lambda_vec(k-1))  

Mbar(1,2) = 1/2 *(1-Lambda_vec(2)*Delta) ;

for k=2:K-1
    if k ~= ms
        Mbar(k,k+1) = 1/2 *(1-Lambda_vec(k+1)*Delta) ; % survival probability from k+1 to k
        Mbar(k,k-1) = 1/2 *(1-Lambda_vec(k-1)*Delta) ; % survival probability from k-1 to k
    else
        Mbar(ms,1)  = Lambda_vec(1)*Delta + 1/2*(1-Lambda_vec(1)*Delta); % from 1 to ms
        for j=2:ms-2
            Mbar(ms,j) = Lambda_vec(j)*Delta ;  % arrivals at ms from position j
        end
        Mbar(ms,ms-1)  = 1/2 *(1-Lambda_vec(ms-1)*Delta) + Lambda_vec(ms-1)*Delta;  % from ms-1  to ms
        Mbar(ms,ms)    = Lambda_vec(ms)*Delta ;
        Mbar(ms,ms+1)  = 1/2 *(1-Lambda_vec(ms+1)*Delta) + Lambda_vec(ms+1)*Delta;  % from ms+1  to ms
        for j=ms+2:K-1
            Mbar(ms,j) = Lambda_vec(j)*Delta ;
        end
        Mbar(ms,K) = Lambda_vec(K)*Delta + 1/2*(1-Lambda_vec(K)*Delta); % from K to ms
    end
end 
Mbar(K,K-1) = 1/2 *(1-Lambda_vec(K-1)*Delta) ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% cross sectional density p
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%% p_{t+Delta}(k) = 1/2 p(k-1)(1-Lambda(k)) + 1/2 p(k-1)(1-Lambda_vec(k)) for k neq ms 
%%% first you get the BM shock, then the Lambda shock
p_vec = zeros(K,1);

T=20000;

% % compute invariant  pbar_vec
% pbar_vec= (Mbar)^(T) *  ptent_vec ; % vector of probabilities (not a density)
% pbar_vec= (Mbar)^(T) *  ones(K,1)/K; % vector of probabilities (not a density)
Mbar_m_eye=(Mbar-eye(K));
Mbar_m_eye(K,:)=ones(1,K) ;
B=zeros(K,1); B(K)=1; 
pbar_vec = Mbar_m_eye\B ; % vector of probabilities (not a density)

% compute N freq of adjust 
N_Delta = sum( pbar_vec .* Lambda_vec) * Delta +  pbar_vec(1)/2 + pbar_vec(end)/2;
N = N_Delta/Delta; % freq per unit of time 

% distribution of size of price changes
q_vec       = Lambda_vec .* pbar_vec * Delta ;
q_vec(1)    = q_vec(1)   +  pbar_vec(1)    /2;
q_vec(end)  = q_vec(end) +  pbar_vec(end)  /2;
q_vec       = q_vec/N_Delta; % vector of probabilities (not a density)

var_Dp=sum(q_vec .* x_vec.^2);

% Exponential Calvo with same variance of Dp as in the quadratic model
q_vec_Calvo= dx*sqrt(1/(var_Dp*2) )* exp(-sqrt(2/var_Dp)*abs(x_vec));

% Golosov Lucas with same variance of Dp as in the quadratic model
q_GL=[-sqrt(var_Dp) sqrt(var_Dp)]; 

% Moments
Var_dp  = sum( x_vec.^2 .* q_vec);
Kurt_dp = sum( x_vec.^4 .* q_vec)/Var_dp^2;
% numerical derivative of pbar
pbar_prime_vec = zeros(K,1);
pbar_prime_vec(2:K-1,1) = (pbar_vec(3:K,1)-pbar_vec(1:K-2,1))/(2*dx) ;
% flexibility index
Flex    = - sum(x_vec .* Lambda_vec .* pbar_prime_vec )  ;

disp( ' ') 
    disp('Parameters');
    disp(['Lambda-0            = ',num2str(Lambda_0,4)]);
    disp(['Lambda-2            = ',num2str(Lambda_2,4)]);
    disp(['nu                   = ',num2str(nu,4)]);
    disp(['sigma                = ',num2str(sigma,4)]);
    disp(' ')
    
    disp( ' ') 
    disp(['dt (Delta)           = ',num2str(Delta,4)]);
    disp(['dx = Delta^.5 sigma  = ',num2str(dx,4)]);
    disp(['N                    = ',num2str(N,4)]); 
    disp(['std_dp               = ',num2str(sqrt(Var_dp),4)]);
    disp(['Kur_dp               = ',num2str(Kurt_dp,4)]);
    disp(['N Var_dp             = ',num2str(Var_dp*N,4)]);
    disp(['sigma^2              = ',num2str(sigma^2,4)]);
    disp(['N Var_dp -sigma^2    = ',num2str(Var_dp*N-sigma^2,4)]);
    disp(['Flex/N               = ',num2str(Flex/N,4)]);
    disp(' ')

    
%%% Transition matrix M until the first price change (w/o reinjections).
%%% We average the values of two adjecent Lambdas to obtain a symmetric M
%%% Symmetry ensures real eigenvalues

M=zeros(K,K);

M(1,2) = 1/2 * (1-1/2*(Lambda_vec(1)+Lambda_vec(2))*Delta) ;
for k=2:K-1
        M(k,k+1) = 1/2 *(1- 1/2*(Lambda_vec(k+1)+Lambda_vec(k))*Delta) ; % arrivals in k from k+1
        M(k,k-1) = 1/2 *(1- 1/2*(Lambda_vec(k-1)+Lambda_vec(k))*Delta) ; % arrivals in k from k-1
end 
M(K,K-1) = 1/2 *(1-1/2*(Lambda_vec(K-1)+Lambda_vec(K))*Delta) ;

% Eigenvalue - eigenvector decomposition
[eig_vec, eig_val_mat] = eig(M);

eig_val         =   diag(eig_val_mat);
proj_minus_x    = - x_vec' * eig_vec ;
proj_pbar_prime =   eig_vec' * pbar_prime_vec ;
proj_1          =   ones(1,K) * eig_vec ;
p_m             =   zeros(K,1);
p_m(ms)         =   1;  
proj_0          =    eig_vec' * p_m ; 

N_T  = 2000;
n_t  = 300;

for j=1:N_T
    n       = n_t * j ;
    L       = diag(eig_val.^n,0) ;
    Y(j)    = proj_minus_x * L * proj_pbar_prime ;
    S(j)    = proj_1  * L * proj_0 ;
    t_vec(j)= n*Delta;
end

h(1)=0;
for j=2:length(S)
    h(j) = -(S(j)-S(j-1))/S(j)/Delta;
end

Mct=zeros(K,K);

M_ct(1,1) =  -1/Delta - Lambda_vec(1);
M_ct(1,2) =  (1/2)/Delta; 
for k=2:K-1
        M_ct(k,k+1) = (1/2)/Delta;
        M_ct(k,k)   = -1/Delta - Lambda_vec(k);
        M_ct(k,k-1) = (1/2)/Delta;
end 
M_ct(K,K)   =     -1/Delta - Lambda_vec(K);
M_ct(K,K-1) =   (1/2)/Delta; 

% or p(t+Delta) = p(t) + Delta M_ct  or p(t+Delta) = 

%  plot   IRF   with GL and Calvo , all with  N=1
 
tnorm_vec=t_vec*N;% Normalized times so that figure corresponds to N=1

% Compute GL IRF for marginal shock
for t=1:length(tnorm_vec)
    Y_GL(t)=0;
    for i=0:30
       Y_GL(t) = Y_GL(t) + 32/((2+4*i)*pi)^2 * exp( -( (2+4*i)*pi)^2 / 8 * tnorm_vec(t)   ) ;
    end
end

figure(1);

pl=plot(tnorm_vec,Y_GL,':',tnorm_vec, Y,'--', tnorm_vec, exp(-tnorm_vec),'k-'); 
set(pl,'Linewidth',2);
ti=title(['Impulse Response Functions  ']);
xl=xlabel('time $t$');
yl=ylabel('Output  $ Y(t)$');
lh=legend('Golosov-Lucas','Quadratic GHF','Calvo');


set(ti,'Interpreter','Latex','Fontsize',20) ;
set(xl,'Interpreter','Latex','Fontsize',20) ; 
set(yl,'Interpreter','Latex','Fontsize',20) ;
set(lh,'Interpreter','Latex','Fontsize',20) ;
axis([0 4 0 1.02]);

% fname = ['fig_three_IRF' num2str(nu)];
% print ('-depsc', fname);

% Compute GL IRF for large  shock
for t=1:length(tnorm_vec)
    Y_GL_big(t)=0;
    for i=0:30
       Y_GL_big(t) = Y_GL_big(t) + 8/((2+2*i)*pi)^2 * exp( -( (2+2*i)*pi)^2 / 8 * tnorm_vec(t)   ) ;
    end
end

% Plot hazards
% Parameters: Lambda(x) = L_0 + L_2_0 /2 x^2
% we also have kappa =  L_2_0 /2


L_0 = Lambda_0 ; 

kappa  = Lambda_2; 
% sigma  = 0.2; % 2.6219 ;

eta = (2*kappa/sigma^2)^(1/4);

N=150;
n_vec=[0:1:N-1];
%%% computes even eignvalues
lambda_e_vec = -eta^2*sigma^2 * (2* n_vec +1/2) ;
%%% compute coefficients of survival function 
beta_S_vec = sqrt(2/pi) * (-1).^n_vec .*  gamma(1/2+n_vec) ./factorial(n_vec) ;

j_vec = (n_vec+1)*2-1 ;
lambda_e_GL_vec = -(j_vec*pi).^2/8;
beta_S_GL_vec   = 4./(j_vec*pi) .* sin(j_vec*pi/2) ;


E_dur = - sum(beta_S_vec./lambda_e_vec);

E_dur_GL = - sum(beta_S_GL_vec./lambda_e_GL_vec);

dt=E_dur/N ;
time_vec=[dt:dt:E_dur*3];

%%% Survival Function S
for j=1:length(time_vec)
    t=time_vec(j);
    S_vec(j)    = sum( beta_S_vec .* exp(lambda_e_vec * t) ) ;
    S_GL_vec(j) = sum( beta_S_GL_vec .* exp(lambda_e_GL_vec * t) ) ;

end


%%% Hazard Rate h
for j=1:length(time_vec)
    t=time_vec(j);
    h_vec(j) = - sum(lambda_e_vec .* beta_S_vec .* exp(lambda_e_vec * t) ) / S_vec(j) ;
    h_GL_vec(j) = - sum(lambda_e_GL_vec .* beta_S_GL_vec .* exp(lambda_e_GL_vec * t) ) / S_GL_vec(j) ;
end

figure(2); 
pl=plot(time_vec,max(h_GL_vec,0),':', time_vec,h_vec,'--',time_vec,ones(1,length(time_vec)),'k-');
set(pl,'Linewidth',2.5);
%ti=title('Quadratic: $\Lambda(x) = \kappa \, x^2, X=\infty$, all cases normalized so $N_a=1$');
% set(ti,'Interpreter','Latex','Fontsize',16) ;
xl=xlabel('Time $t$ (duration)');
yl=ylabel('Hazard rate $h(t)$');
le = legend('Golosov-Lucas','Quadratic GHF','Calvo'); 
hline(-lambda_e_vec(1),'r');
hline(-lambda_e_GL_vec(1),'b');
%hline(1/E_dur,'k','Calvo, N');
%vline(E_dur,'g');
axis([0 2*E_dur 0 -1.1*lambda_e_vec(1)]);
set(xl,'Interpreter','Latex','Fontsize',18) ; 
set(yl,'Interpreter','Latex','Fontsize',18) ;
set(le,'Interpreter','Latex','Fontsize',16,'Location','SouthEast') ;



